Step4 <- function(ptild, Sk, pik, Addj, T0) {
  #  updates the support Sk if necessary
  #
  # INPUT
  # ptild : vector of empirical probabilities
  # Sk : vector. Support of pi.
  # pik : vector with the same length as Spi. Values for the positive
  #       measure
  # Addj : point added to the support at Step2
  # T0 : matrix of j-th triangular functions for
  # j varying from 1 to L
  #
  # OUTPUT
  # code : scalar
  #  code = 0. The support Sk and the values pik has been successfully
  #            updated
  #
  #  code =1. It was not possible to find a new support.
  #
  # If code=0,
  #  NewS : the new support
  #  Newpi : values of the positive measure
  #
  # If code=1,
  #  Trace : nature of the problem
  # 
  cond=0
  Sk1=Sk
  pik1=pik
  M <- length(ptild)
  while (cond==0) {
    m <- max(c(Sk1, Addj))
    np <- length(c(Sk1, Addj))
    if (m >= M) {
      emp <- rep(0,m)
      emp[1:M] <- ptild
    # projection on the new support
      lesT <-t(T0[c(Sk1, Addj),1:m])
    }
    if (m < M) {
      emp <- ptild
      lesT <-t(T0[c(Sk1, Addj),1:M])
    }  
    if (np==1) reg <- lm(emp~t(lesT)-1)
    if (np>1) reg <- lm(emp~lesT-1)
    if (reg$coeff[np] < 0) {
      cond=1
      trace=paste("Step2 added",Addj,
        "Step 4 : its coefficient is negative:",reg$coeff[indNeg])
      return(list(code=1,trace=trace))
    }
    # if not
    if (reg$coeff[np] >= 0) {
      indNeg <- reg$coeff[1:(np-1)]<0
      # if the coefficients corresponding to Sk1 are all positive
      # then return to Estim to continue the algorithm
      if (sum(indNeg) ==  0) {
        cond=1
        return(list(code=0,NewS=c(Sk1,Addj),Newpi= reg$coeff))
      }
      # if some coefficients corresponding to Sk1 are negative
      # then suppress one of them
      if (sum(indNeg) >= 1) {
        if (np==2) {
          Sk1 <- NULL
          pik1 <- NULL
        }
        if (np>2) {
          amoinsb  <- pik1-reg$coeff[-np]
          ii <- which(amoinsb>0)
          l <- which.min(pik1[ii]/amoinsb[ii])
          Sk1 <- Sk1[-ii[l]]
          pik1 <- pik1[-ii[l]]
        }
      }
      # and calculate the projection on this new support
    }
  }
}


Step2 <- function(emp,Spi,pi,T0,t.zero){
  # calculates the directional derivatives D
  # and add a point to the support of pi if necessary
  #
  # INPUT
  # emp : vector of dimension L, whose M+1 first coordinates equal ptild
  # Spi : vector. Support of pi. max(Spi)=L
  # pi : vector with the same length as Spi. Values for the positive measure
  # The L-th trinagular function
  #
  # OUTPUT
  # D vector with L components. Directional derivatives of Psi
  # Addj : scalar. Addj = min(D). If  Addj <0, the point Addj must be
  # added to the support of pi
#  m <- max(Spi)
  m=dim(T0)[1]
  D <- rep(0,m)
  # calculation of the directional derivatives
  # D[j], for j varying from 1 to L
  for(j in 1:m){
    d <- rep(0,j) 
    for(l in 1:j){
      d[l] <- T0[j,l]*(sum(pi*T0[Spi,l])-emp[l])
    }
    D[j] <- sum(d)                        
  }
  # mise a 0 des valeurs inferieures au seuil
  if(sum(abs(D) < t.zero) != 0 ){ D[abs(D) < t.zero] <- 0}
  # If there exists at least one D[j] negative, then
  # add to the support Spi the value of j that minimizes D.
  minD <- min(D)
  if (minD >=0) Addj=0
  if (minD <0) Addj <- which.min(D)
  return( list(Addj=Addj, D=D) )
}
#

#$$$$$ 

# mini() fournit la solution des moindres carres pi.estim sur 
# le nouveau support S

#$$$$$




Tri1 <- function(J){
  # calculation of the matrix of j-th triangular functions for
  # j varying from 1 to J
  #
  # INPUT
  # J scalar
  #
  # OUTPUT
  # t : matrix with J rows and J columns
  # t(j,l) = 2*(j-(l-1))/(j*(j+1)) for 1<=l<=j
  t <- matrix(0,J,J)
  for(j in 1:J){
    t[j,1:j] <- 2*(j-(0:(j-1)))/(j*(j+1))
  }
  dimnames(t) <- list(paste("j=",1:J,sep=""),
                   paste("l=",0:(J-1),sep=""))
  return(t)
}

Estim <- function(L, ptild, t.zero){
  # calculation of pHat when the max(Supp(pi))=L
  #
  # INPUT
  # L : scalar : max of the support of pi
  # ptild : vector of empirical probabilities
  # t.zero : non-zero minimum value
  #
  # OUTPUT
  # code : integer
  #        code=0 : the convergence is reached
  #        code=1 : the convergence is not reached
  #                 (no positive measure is found)
  #  Spi : vector of integers.  Support of pi
  #  pi : vector with the same length as Spi.
  #       Values of the positive measure
  #  p : vector with L components. Values of the convex function p
  #  SumP : scalar. Sum of the components of p
  #  Psi : scalar. value of the crierion Psi
  #  trace : if code=1, nature of the problem
  emp <- rep(0,L)
  M <- length(ptild)
  emp[1:M] <- ptild
  iter <- 0
  # Initial value for (pi, Spi) : the L-th triangular function
  #
  # calculation of the matrix of j-th triangular functions for
  # j varying from 1 to L
  T0 <- Tri1(L)
  pi0 <-sum(emp*T0[L,])/sum(T0[L,]**2)
  Spi0 <- c(L)
  p0 <- pi0*T0[L,]
  Psi0 <- sum(p0^2)/2-sum((emp[1:length(p0)])*p0)
  SumP0 <- sum(p0)
  # The function Step2 calculates the directional derivatives D
  # and add a point to the support of pi if necessary
  St2 <- Step2(emp,Spi0,pi0,T0,t.zero)
  #
  # If St2$Addj==0 then the convergence is reached
  # when the max(Supp(pi))=L
  if (St2$Addj==0) {
   return(list(code=0, Spi=Spi0, pi=pi0, p=p0, Psi=Psi0, SumP=SumP0))
  }
  #
  # If St2$Addj !=0
  # Add one point to the support, and update the support in order to
  # get a positive measure
  Sk = Spi0
  pik = pi0
  pk=p0
  code=1
  while(code==1 ) {
    # the function Step4 updates the support Sk if necessary
    St4 <- Step4(ptild, Sk, pik, St2$Addj, T0)
     # if St4$code=1, then return to the main function for continuing
    # with a new value of L
    if (St4$code >=1 ) {
      return(list(code=St4$code, trace=St4$trace,
                  Spi=Spi0, pi=pi0, p=p0, Psi=Psi0, SumP=SumP0))
    }
    # if not, then go to step 2 with the new positive measure
    if (St4$code==0) {
      Sk=St4$NewS
      pik=St4$Newpi
      St2 <- Step2(emp,Sk,pik,T0,t.zero)
      # if St2$Addj=0, all the derivative are positive and the minimum
      # of Psi is reached (on the support with maximum smaller than L)
      if (St2$Addj==0) {
        code=0
        p = matrix(pik,nrow=1,ncol=length(pik))%*%T0[Sk,1:max(Sk)]
        Psi <- sum(p^2)/2-sum((emp[1:length(p)])*p)
        SumP <- sum(p)
        return(list(code=code, Spi=Sk, pi=pik, p=p, Psi=Psi, SumP=SumP))
      }
    }
  }
}


pConv <- function(ptild,  t.zero=1e-10, t.P=1e-8, max.sn=NULL){
  # Calculation of pHat with support 0:LHat
  #
  # INPUT
  # ptild : vector of empirical probabilities
  #         with M+1 components
  #         where M=max(Support(ptild))
  # t.zero : non-zero minimum value
  # t.P : threshold for testing that pHat  sums to one
  # max.sn : maximum of the support of pHat
  #          By default max.sn=10*M
  #
  # OUTPUT
  # res : list with components
  #   cvge = 0 if the criterion Psi decreases with the support of pi
  #   cvge = 1 if Psi increases
  #   cvge = 2 if maximum number of iterations reached
  #   code
  #   Spi : Support of the positive measure pi at the last iteration
  #   pi : values of the positive measure pi at the last iteration
  #   p : values of pHat
  #   Psi : scaler value of the criterion to be minimised
  #   sumP : scalar = sum(pHat) at convergence
  # history : data frame with components
  #   L : the maximum of the support of pi
  #   Psi : value of the criterion for the value L
  #   SumP : value of sum(pHat)
  M <- length(ptild)-1
  if (is.null(max.sn)) max.sn=10*M
  L1=M+1
  Psi <- NULL
  SumP <- NULL
  code <- NULL
  i=0
  for (L in L1:max.sn) {
    i=i+1
    if (i>=2) res1=resL
    resL <- Estim(L, ptild, t.zero)
    if (resL$code>0) {
      code=c(code,resL$code)
      Psi <- c(Psi,resL$Psi)
      SumP <- c(SumP,resL$Psi )
      if (L==max.sn) {
        trace=paste("The maximum number of iterations, max.sn=",
          max.sn,
          " is reached. The convergence is not reached with Code=",
          resL$code)
        history <- data.frame(L=L1:(L1+i-1),code=code,Psi=Psi,SumP=SumP)
        pi <- resL$pi
        names(pi) <- paste("j=",resL$Spi,sep="")
        pi <- pi[order(resL$Spi)]
        Spi <- sort(resL$Spi)
        res <- list(cvge=2,trace=trace,history=history,
                    Spi=Spi,pi=pi,p=resL$p,
                    Psi=resL$Psi,SumP=resL$SumP)
        return(res)
      }
    }
    if (resL$code ==0) {
      code=c(code,resL$code)
      Psi <- c(Psi,resL$Psi)
      SumP <- c(SumP,resL$SumP)
      if ((i==1)&(abs(resL$SumP-1)<t.P)) {
        history <- data.frame(L=L1,code=code,Psi=Psi,SumP=SumP)
        trace="Convergence is reached"
        res <- list(cvge=0,trace=trace,history=history,
                    Spi=resL$Spi,pi=resL$pi,p=resL$p,
                    Psi=resL$Psi,SumP=resL$SumP)
        return(res)
      }
      if (i>1) {
        if (abs(resL$SumP-1)<t.P) {
          history <- data.frame(L=L1:(L1+i-1),code=code,Psi=Psi,SumP=SumP)
          trace="Convergence is reached"
          pi <- resL$pi
          names(pi) <- paste("j=",resL$Spi,sep="")
          pi <- pi[order(resL$Spi)]
          Spi <- sort(resL$Spi)
          res <- list(cvge=0,trace=trace,history=history,
                      Spi=Spi,pi=pi,p=resL$p,
                      Psi=resL$Psi,SumP=resL$SumP)
          return(res)
        }
        DeltaPsi <-Psi[length(Psi)]-Psi[length(Psi)-1]
        if (DeltaPsi > 0) {
          trace=paste("The algoritm stopped because for L=",L,
            ",the criterion increases. Psi(",L-1,")=",
            Psi[length(Psi)-1],"Psi(",L,")=",Psi[length(Psi)])
          history <- data.frame(L=L1:(L1+i-1),code=code,Psi=Psi,SumP=SumP)
          pi <- res1$pi
          names(pi) <- paste("j=",res1$Spi,sep="")
          pi <- pi[order(res1$Spi)]
          Spi <- sort(res1$Spi)
          res <- list(cvge=1,trace=trace,history=history,
                      Spi=Spi,pi=pi,p=res1$p,
                      Psi=res1$Psi,SumP=res1$SumP)
          return(res)
        }
      }
    }
  }
}
