rm(list=ls())
source("Simul.R")
source("EST.VSH.R")


# Example 1
# one triangular function T_j, for j=20
n=25
Supp=20
p <- pmelTriang(Supp)
X <-rmelTriang(Supp, size= n)
ptildeX <- pChap(X)

res <- pConv(ptildeX$freq)

plot(c(0,max(c(max(Supp),res$pi))),range(c(p,ptildeX$freq,res$p)),type="n",
       xlab="counts",ylab="frequencies")
points(ptildeX$supp,ptildeX$freq,pch=4,col=3, lwd=2)
points(0:(length(p)-1), p,pch=8,col=2)
points(0:(length(res$p)-1), res$p,pch=16,col=4)
legend(x=mean(ptildeX$supp),y=max(res$p),pch=c(4,8,16),col=c(3,2,4),
       legend=c("ptilde","p","pHat"))


# Example 2
# mixture of 3 triangular functions
n=50
Supp <- sample(1:25,3)
proba <- sample(1:100,3)/100

p <- pmelTriang(Supp, proba=proba)
X <-rmelTriang(Supp, proba=proba, size= n)
ptildeX <- pChap(X)

res <- pConv(ptildeX$freq)
plot(c(0,max(c(max(Supp),res$pi))),range(c(p,ptildeX$freq,res$p)),type="n",
       xlab="counts",ylab="frequencies")
points(ptildeX$supp,ptildeX$freq,pch=4,col=3, lwd=2)
points(0:(length(p)-1), p,pch=8,col=2)
points(0:(length(res$p)-1), res$p,pch=16,col=4)
legend(x=mean(ptildeX$supp),y=max(res$p),pch=c(4,8,16),col=c(3,2,4),
       legend=c("ptilde","p","pHat"))

# the same with n=500
n=500
X <-rmelTriang(Supp, proba=proba, size= n)
ptildeX <- pChap(X)

res <- pConv(ptildeX$freq)
plot(c(0,max(c(max(Supp),res$pi))),range(c(p,ptildeX$freq,res$p)),type="n",
       xlab="counts",ylab="frequencies")
points(ptildeX$supp,ptildeX$freq,pch=4,col=3, lwd=2)
points(0:(length(p)-1), p,pch=8,col=2)
points(0:(length(res$p)-1), res$p,pch=16,col=4)
legend(x=mean(ptildeX$supp),y=max(res$p),pch=c(4,8,16),col=c(3,2,4),
       legend=c("ptilde","p","pHat"))

# Simulation for comparing ptilde and pHat

nSim=500

n=25
Supp=20
p <- pmelTriang(Supp)

result <- matrix(nrow=nSim,ncol=3)
dimnames(result)[[2]] <- c("cvge","r.emp","r.Hat")
for (i in 1:nSim) {
  X <-rmelTriang(Supp, size= n)
  f.emp <- pChap(X)$freq
  res <- pConv(f.emp)
  m <- max(c(Supp,length(f.emp),res$Spi))
  pV=c(p,rep(0,m-length(p)))
  pHat=c(res$p,rep(0,m-length(res$p)))
  f.emp=c(f.emp,rep(0,m-length(f.emp)))
  result[i,] <- c(res$cvge,sum((pV-f.emp)**2),
                  sum((pV-pHat)**2))
}

apply(result,2,mean)
#       cvge       r.emp       r.Hat 
#0.000000000 0.036914641 0.004654442 

